/*
 * Test pipes.
 *
 * Copyright (C) 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>

#include <stdio.h>
#include <string.h>

#include <fsclient/client.h>



static void show(char *s, unsigned long length)
{
  char buffer[length + 1];

  memcpy(buffer, s, length);
  buffer[length] = '\0';
  printf("Received (%ld): %s\n", length, buffer);
}

int main(void)
{
  file_descriptor_t *reader, *writer;
  char buffer[128];
  ssize_t nread, tread;
  ssize_t nwritten;

  if (client_pipe(&reader, &writer))
  {
    printf("Failed to open pipe.\n");
    return 1;
  }

  printf("Read pipe: %ld\n", reader->obj.buffer.size);
  printf("Write pipe: %ld\n", writer->obj.buffer.size);

  printf("Read address: %p\n", reader->obj.buffer.addr);
  printf("Write address: %p\n", writer->obj.buffer.addr);

  nwritten = 0;
  while (nwritten < 8192)
  {
    memset(buffer, (int) 'a' + ((nwritten / 128) % 26), 128);
    nwritten += client_write(writer, buffer, 128);
  }

  printf("Written: %ld\n", nwritten);
  client_flush(writer);

  tread = 0;

  do
  {
    nread = client_read(reader, buffer, 128);
    show(buffer, nread);
    tread += nread;
  }
  while (nread);

  printf("Read: %ld\n", tread);

  client_close(reader);
  client_close(writer);

  printf("Terminating.\n");
  return 0;
}
